//
// This class builds the Services as Cylinders
//
// The services are grouped in 4 groups (as in G3)
// I keep this grouping as it natural in the database.
// They are Barrel-Inside tracking volume
//          Barrel-Outside tracking volume
//          Endcap-Inside tracking volume
//          Edncap-Outside tracking volume
//
// The elements for each group are defined in initialize,
// with Barrel/endcap coming from the geometry manager and inside oputside,
// from the mother volume name.
// This class builds the n-th  service element, with n defined in SetCylinder
//
//
//
// The information on which layer to build is retrieved by the Geometry Manager
//
// Contained by: passed by argument
//
// Contains: nothing
//

#include "PixelGeoModel/GeoPixelServices.h"
#include "GeoModelKernel/GeoTube.h"
#include "GeoModelKernel/GeoLogVol.h"
#include "GeoModelKernel/GeoPhysVol.h"
#include "GeoModelKernel/GeoMaterial.h"

#include <sstream>

GeoPixelServices::GeoPixelServices(std::string root) {
  //
  // Initialize the service geometry properly accordning to the
  // mother volume where it goes.
  // Of course services cannot be placed in other volumes.
  if(root == "/Pixel") {
    initialize("Outside");
    m_zone = "Out";
  } else if (root == "/Pixel/PixelBarrel" || root == "/Pixel/PixelEndcap" ) {
    initialize("Inside");
    m_zone = "In";
  } else {
    std::cerr << "Services can  not be placed in this volume!!!! " << root << std::endl;
  }
}

void GeoPixelServices::initialize(std::string a)
{
  //
  // Here I inizialize the geometry information contained in the vectors
  //

  //
  // Loop over the service elements:
  //
  int NServices =  gmt_mgr->PixelServiceNFrame(a);
  for(int ii = 0; ii<NServices ; ii++) {
    //
    // If the element is attached to a layer/disk check if the LD is present.
    //
    int LD = gmt_mgr->PixelServiceLD(a, ii);
    //
    // C counts from zero. So the LDs are 0,1,2. -1 correspond to a
    // service which is not attached to any LD.
    //
    if(LD != -1) gmt_mgr->SetCurrentLD(LD);
    if(LD ==-1 || gmt_mgr->isLDPresent() == true) {
      double zpos,halflength;
//
// Retrieve/calculate the parameters for the volume.
//
// Here the atlsim logic has to be interpreted, as this is going into
// the structure of the PBOI bank that I'm using
//
      std::string mat = gmt_mgr->PixelServiceMaterial(a, ii);
      double* r = gmt_mgr->PixelServiceR(a, ii);
      double* z = gmt_mgr->PixelServiceZ(a, ii);
      //
      // If zin is 0... (within 10^-5) this is a volume symmetric around
      // the origin
      //
      if(fabs(z[0]) < 0.000001) {
        zpos = 0.;
        halflength = fabs(z[1])-m_epsilon;
      } else {
          halflength = fabs(z[1]-z[0])/2.-m_epsilon;
          zpos = (z[1]+z[0])/2.+m_epsilon/2.;
      }
      //
      // Push the object into the vectors: (twice if needed)
      //
      //
      //
      bool twovolumes;
      if(zpos > 0.00001 && r[0] >0.) {
        twovolumes = true;
      } else {
        twovolumes = false;
      }
      // only once for the inside endcaps as I already have two endcaps!
      if(gmt_mgr->isEndcap() && a == "Inside") twovolumes = false;
      _rmin.push_back(fabs(r[0]));
      _rmax.push_back(r[1]);
      _zpos.push_back(zpos);
      _halflength.push_back(halflength);
      material.push_back(mat);
      if(twovolumes) {
        _rmin.push_back(fabs(r[0]));
        _rmax.push_back(r[1]);
        _zpos.push_back(-zpos);
        _halflength.push_back(halflength);
        material.push_back(mat);
      }
      delete[] r;
      delete[] z;
    }
  //
  // Delete r and z...
  //
  }
}

GeoVPhysVol* GeoPixelServices::Build( ) {
  GeoMaterial* serviceMat = mat_mgr->getMaterial(material[_nframe]);
  //
  // Dimensions: (_nframe set with teh SetCylinder method)
  //
  double rmin = _rmin[_nframe];
  double rmax = _rmax[_nframe];
  double halflength = _halflength[_nframe];
  const GeoTube* serviceTube = new GeoTube(rmin,rmax,halflength);
  std::string logName;
  if(gmt_mgr->isBarrel() ) logName = m_zone+"Brl";
  else logName = m_zone+"EC";
  std::ostringstream o;
  o << _nframe;
  logName = logName+o.str();
  GeoLogVol* theService = new GeoLogVol(logName,serviceTube,serviceMat);
  GeoPhysVol* servicePhys = new GeoPhysVol(theService);
  return servicePhys;
}

