
#include "PixelGeoModel/PixelDetectorFactoryDC2.h"

// Envelope, as a starting point of the geometry
//#include "PixelGeoModel/GeoPixelEnvelope.h"
#include "PixelGeoModel/PixelDetectorDC1DC2.h"
#include "PixelGeoModel/PixelSwitches.h" 

// GeoModel includes
#include "GeoModelKernel/GeoNameTag.h"  
#include "GeoModelKernel/GeoPhysVol.h"  
#include "GeoModelKernel/GeoAlignableTransform.h"  

// StoreGate includes
#include "StoreGate/StoreGateSvc.h"
#include "StoreGate/DataHandle.h"

// For the Msg Svc
#include "AthenaKernel/getMessageSvc.h"
#include "GaudiKernel/MsgStream.h"

// InDetReadoutGeometry
#include "InDetReadoutGeometry/SiCommonItems.h" 


using InDetDD::PixelDetectorManager; 
using InDetDD::SiCommonItems; 

using namespace PixelGeoDC2;


PixelDetectorFactoryDC2::PixelDetectorFactoryDC2(StoreGateSvc *detStore, const PixelSwitches & switches)
  :m_detectorManager(NULL),
   m_detectorStore(detStore)
{
  // Create the detector manager
  m_detectorManager = new PixelDetectorManager(detStore);
  GeoVPixelFactory::SetDDMgr(m_detectorManager);

  // Create the geometry manager.
  PixelGeometryManager * geometryManager =  PixelGeometryManager::GetPointer();
  
  // Pass the switches
  geometryManager->SetServices(switches.services());
  geometryManager->SetG3CompatibleDigits(switches.g3CompatibleDigits());
  geometryManager->SetDC1Geometry(switches.dc1Geometry());
  geometryManager->SetAlignable(switches.alignable());
  geometryManager->SetInitialLayout(switches.initialLayout());
  geometryManager->setUseMagneticFieldSvc(switches.useMagneticFieldSvc());

  // Create SiCommonItems ans store it in geometry manager. 
  // These are items that are shared by all elements
  SiCommonItems * commonItems = new SiCommonItems(m_detectorManager->getIdHelper());
  if (!geometryManager->useMagneticFieldSvc()) commonItems->disableMagneticFieldSvc();
  geometryManager->setCommonItems(commonItems);
 
  // Determine if initial layer and tag from the id dict are consistent
  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactoryDC2");
  bool initialLayoutIdDict = (m_detectorManager->tag() == "initial_layout");
  if (geometryManager->InitialLayout() != initialLayoutIdDict ) {
      log << MSG::WARNING << "IdDict tag is \"" << m_detectorManager->tag() 
	  << "\" which is inconsistent with the layout choosen!"
	  << endreq;
  } 

  
  //
  // Set Version information
  //
  std::string versionTag = geometryManager->versionTag();
  std::string versionName = "DC2";
  std::string layout = "Final";
  std::string description = "DC2 Geometry";
  int versionMajorNumber = 2;
  int versionMinorNumber = 2;
  int versionPatchNumber = 0;

  if (geometryManager->G3CompatibleDigits()) {
    description += ", G3 Compatible Digits";
    versionMinorNumber = 1;
  }

  if (geometryManager->InitialLayout()) {
    layout = "Initial";
  }
  
  // We determine if we are running DC1 geoemtry via 
  // Barrel version number in NOVA 
  if (geometryManager->DC1Geometry()) {
     versionName = "DC1";
     description = "DC1 Geometry (300um B-Layer pixels)";
     versionMajorNumber = 1;
     versionMinorNumber = 2;
     if (geometryManager->G3CompatibleDigits()) {
       description += ", G3 Compatible Digits";
       versionMinorNumber = 1;
     }
  }    

  InDetDD::Version version(versionTag,
			   versionName, 
			   layout, 
			   description, 
			   versionMajorNumber,
			   versionMinorNumber,
			   versionPatchNumber);
  m_detectorManager->setVersion(version);

}


PixelDetectorFactoryDC2::~PixelDetectorFactoryDC2()
{

}



//## Other Operations (implementation)
void PixelDetectorFactoryDC2::create(GeoPhysVol *world)
{
  PixelGeometryManager * geometryManager =  PixelGeometryManager::GetPointer();

  MsgStream log(Athena::getMessageSvc(), "PixelDetectorFactoryDC2");

  log << MSG::INFO << "Building Pixel Detector" << endreq;
  log << MSG::INFO << " " << m_detectorManager->getVersion().fullDescription() << endreq;

  // Printout the parameters that are different in DC1 and DC2.
  log << MSG::INFO << " B-Layer basic eta pitch: " << geometryManager->DesignPitchZ(true)/micrometer << "um" << endreq;  
  geometryManager->SetCurrentLD(0);
  geometryManager->SetBarrel();
  log << MSG::INFO << " B-Layer sensor thickness: " << geometryManager->PixelBoardThickness()/micrometer << "um" << endreq;   
  
  //
  // Create the Pixel Envelope...
  GeoPixelEnvelope pe;
  GeoVPhysVol* pephys = pe.Build() ;
  GeoAlignableTransform * transform = new GeoAlignableTransform(HepTransform3D());
  
  //
  // Add this to the world
  //
  GeoNameTag *tag = new GeoNameTag("Pixel");         
  world->add(tag);
  world->add(transform);
  world->add(pephys);

  // Store alignable transform
  Identifier id = m_detectorManager->getIdHelper()->wafer_id(0,0,0,0);
  m_detectorManager->addAlignableTransform(2, id, transform, pephys);

  //
  // Add this to the list of top level physical volumes:             
  //
  m_detectorManager->addTreeTop(pephys);                                       

  
  // Initialize the neighbours
  m_detectorManager->initNeighbours();

  // Register the callbacks and keys and the level corresponding to the key.
  if (geometryManager->Alignable()) {
    m_detectorManager->addKey("/Indet/Align/ID",2);
    m_detectorManager->addKey("/Indet/Align/PIX",1);
    m_detectorManager->addKey("/Indet/Align/PIXB1",0);
    m_detectorManager->addKey("/Indet/Align/PIXB2",0);
    m_detectorManager->addKey("/Indet/Align/PIXB3",0);
    m_detectorManager->addKey("/Indet/Align/PIXEA1",0);
    m_detectorManager->addKey("/Indet/Align/PIXEA2",0);
    m_detectorManager->addKey("/Indet/Align/PIXEA3",0);
    m_detectorManager->addKey("/Indet/Align/PIXEC1",0);
    m_detectorManager->addKey("/Indet/Align/PIXEC2",0);
    m_detectorManager->addKey("/Indet/Align/PIXEC3",0);
  }
}

const PixelDetectorManager * PixelDetectorFactoryDC2::getDetectorManager() const
{
  return m_detectorManager;
}

