//
// Pixel Sensitive Detector.
// The Hits are processed here. For every hit I get the position and
// an information on the sensor in which the interaction happened
//
#include <fstream>
#include "FadsSensitiveDetector/SensitiveDetectorEntryT.hh"
#include "FadsSensitiveDetector/SensitiveDetectorCatalog.hh"



// G4Pixel 
#include "PixelG4_SD/PixelSensorSD.h"

// Geant4 Stuff
#include "G4HCofThisEvent.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"

// CLHEP transform
#include "CLHEP/Geometry/Transform3D.h"    
// Units
#include "CLHEP/Units/SystemOfUnits.h"    

// to permit access to StoreGate
#include "GaudiKernel/ISvcLocator.h"
#include "GaudiKernel/Bootstrap.h"
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/IMessageSvc.h"
#include "StoreGate/StoreGateSvc.h"

#include "SimHelpers/DetectorGeometryHelper.h"
#include "MCTruth/TrackHelper.h"

#define PIXELSD_DEBUG

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

//////////////////////////////////////////////////////////////////////////////////
// Initialize static data 
///////////////////////////////////////////////////////////////////////////////

static SensitiveDetectorEntryT<PixelSensorSD> mdtsd("PixelSensorSD");



PixelSensorSD::PixelSensorSD(G4String name) :  FadsSensitiveDetector(name)
{
  //
  // Get StoreGate pointers:
  // 
  ISvcLocator* svcLocator = Gaudi::svcLocator(); // from Bootstrap
  StatusCode status = svcLocator->service("MessageSvc", m_msgSvc);
  MsgStream log(m_msgSvc, "PixelSensorSD");
  if (status.isFailure()) {
    log << MSG::FATAL << "MessageSvc not found !" << endreq;
  } else {
    log << MSG::DEBUG << "MessageSvc initialized." << endreq;
  }
  status = svcLocator->service("StoreGateSvc", m_sgSvc);
  if (status.isFailure()) {
    log << MSG::FATAL << "StoreGateSvc  not found !" << endreq;
  } else {
    log << MSG::DEBUG << "StoreGateSvc initialized." << endreq;
  }
}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PixelSensorSD::~PixelSensorSD(){ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PixelSensorSD::Initialize(G4HCofThisEvent* HCE)
{
  MsgStream log(m_msgSvc, "PixelSensorSD");
  log << MSG::DEBUG << "Initializing SD" << endreq;
  DetectorGeometryHelper DGHelp;
  if(  DGHelp.GeometryType("Pixel") == GeoModel )
  {
         m_isGeoModel = true;
         log << MSG::INFO << "Pixel Geometry is from GeoModel" << endreq;
  } else {
         m_isGeoModel = false;
         log << MSG::INFO << "Pixel Geometry is from pure G4" << endreq;
  }
  //
  // Create a fresh map to store the hits
  //
  myHitColl = new SiHitCollection("PixelHits");

}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4bool PixelSensorSD::ProcessHits(G4Step* aStep,G4TouchableHistory* ROhist)
{
  MsgStream log(m_msgSvc, "PixelSensorSD::ProcessHits");
    log << MSG::VERBOSE << "Process Hit" << endreq;


  G4double edep = aStep->GetTotalEnergyDeposit();
  edep *= MeV;
  if(edep==0.) {
        if(aStep->GetTrack()->GetDefinition()->GetParticleType()!="geantino") return false;
  }
  //use the global time. i.e. the time from the beginning of the event
  // 
  // Get the Touchable History:
  //
  G4TouchableHistory*  myTouch = (G4TouchableHistory*)(aStep->GetPreStepPoint()->GetTouchable());
#ifdef PIXELSD_DEBUG
     for (int i=0;i<myTouch->GetHistoryDepth();i++)
     {
         std::string detname=myTouch->GetVolume(i)->GetLogicalVolume()->GetName();
         int copyno=myTouch->GetVolume(i)->GetCopyNo();
         log << MSG::VERBOSE << "Volume "<<detname <<" Copy Nr. "<<copyno<<endreq;
     }
#endif

  //
  // Get the hit coordinates. Start and End Point
  //
  G4ThreeVector coord1 = aStep->GetPreStepPoint()->GetPosition();
  G4ThreeVector coord2 = aStep->GetPostStepPoint()->GetPosition();

  // Calculate the local step begin and end position. 
  // From a G4 FAQ:
  // http://geant4-hn.slac.stanford.edu:5090/HyperNews/public/get/geometry/17/1.html
  //
  const G4AffineTransform transformation = myTouch->GetHistory()->GetTopTransform();
  G4ThreeVector localPosition1 = transformation.TransformPoint(coord1);
  G4ThreeVector localPosition2 = transformation.TransformPoint(coord2);

  HepPoint3D lP1,lP2;
  lP1[SiHit::xEta] = localPosition1[2]*mm;
  lP1[SiHit::xPhi] = localPosition1[1]*mm;
  lP1[SiHit::xDep] = localPosition1[0]*mm;

  lP2[SiHit::xEta] = localPosition2[2]*mm;
  lP2[SiHit::xPhi] = localPosition2[1]*mm;
  lP2[SiHit::xDep] = localPosition2[0]*mm;

  //
  // Get it into a vector in local coords and with the right units:


  // Now Navigate the history to know in what detector the step is:
  // and finally set the ID of det element in which the hit is.
  //
  G4int History;
  //
  // Is it TB, barrel or endcap?
  //
  int BrlEcap = 0;
  int LayerDisk = 0;
  int etaMod = 0;
  int phiMod = 0;
  int side = 0;

  //   G4VPhysicalVolume* TheVolume = myTouch->GetVolume();
  //
  // In the case of the TB the positioning integers won't be initialized
  // and the identifying integer will be zero. There is no need to do 
  // anything else

  int BEcopyNo =  myTouch->GetVolume()->GetCopyNo();

  // Barrel
  if(BEcopyNo == 100) {
    // Move up of one level in the tree to get the module number:
    //
    int ModuleNumber = myTouch->GetVolume(1)->GetCopyNo();
    etaMod = ModuleNumber;
    //
    // Another level for the ladder
    //
    int LadderNumber = myTouch->GetVolume(2)->GetCopyNo();
    phiMod = LadderNumber;
    //
    // Another one for the layer
    //
    int LayerNumber = myTouch->GetVolume(3)->GetCopyNo();
    LayerDisk = LayerNumber;
    //
    // Some printout
    //
    log << MSG::VERBOSE << "In the Pixel Barrel" << endreq; 
    log << MSG::VERBOSE << "----- Module # " << ModuleNumber << endreq; 
    log << MSG::VERBOSE << "----- Ladder # " << LadderNumber << endreq; 
    log << MSG::VERBOSE << "----- Layer #  " << LayerNumber << endreq; 

  } 

  //EndCap
  if(BEcopyNo == 200) {

    // Use the copy no. to get the id:
    int SubDiskNumber = myTouch->GetVolume(1)->GetCopyNo();
    if (SubDiskNumber == 48) SubDiskNumber = 0;
    phiMod = SubDiskNumber;

    // Move up of one level in the tree to get the disk number:
    int DiskNumber = myTouch->GetVolume(2)->GetCopyNo();
    LayerDisk = DiskNumber;

    // Move up of one level in the tree to get the EC number:
    int ECNumber = myTouch->GetVolume(3)->GetCopyNo();
    BrlEcap = ECNumber;
    if(BrlEcap == -2) BrlEcap = 1;

    // Some printout
    log << MSG::VERBOSE << "In the Pixel EndCap" << endreq; 
    log << MSG::VERBOSE << "----- SubDisk # " << SubDiskNumber << endreq; 
    log << MSG::VERBOSE << "----- Disk #    " << DiskNumber << endreq; 
    log << MSG::VERBOSE << "----- EC #      " << BrlEcap << endreq;     
  }
  // get the barcode from the track helper
  TrackHelper trHelp(aStep->GetTrack());
  int barcode = trHelp.GetBarcode();
  SiHit* newHit = new SiHit(lP1,
			    lP2,
			    edep,
			    aStep->GetPreStepPoint()->GetGlobalTime(),
			    barcode,
			    0,BrlEcap,LayerDisk,etaMod,phiMod,side);
  myHitColl->Insert(newHit );
  return true;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PixelSensorSD::EndOfEvent(G4HCofThisEvent* HCE)
{
  //  m_sgSvc->record(myHitColl,"PixelHits");  
  //   myHitColl->Export();
  m_hitCollHelp.ExportCollection<SiHitCollection>(myHitColl);
  MsgStream log(m_msgSvc, "PixelSensorSD");
  log << MSG::DEBUG << "End of event " << endreq;
}


